/*
 * Copyright 2023 NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 */
#include "fsl_debug_console.h"
#include "fsl_flash.h"
#include "fsl_flash_ffr.h"
#include "fsl_common.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "board.h"

#include "fsl_clock.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define SECTOR_INDEX_FROM_END 2U /* start from the last 2 page*/
#define FLASH_BUFFER_LEN 512
#define REMAP_IMAGE_SIZE (0x8000)
flash_config_t FlashConfig1 = {0u};
flash_config_t FlashConfig2 = {0u};
/*******************************************************************************
 * Prototypes
 ******************************************************************************/
//!@brief Run Intercal FLASH CDPA test and return the test result
// CDPABlock means which BLK_CFG_W needs to be changed,CDPAPageId means which region needs to be changed
bool flash_CDPA_test(uint32_t *CDPABlock, uint32_t CDPAPageId);
//!@brief Run Intercal FLASH hiding test and return the test result
// MBCAddr means which BLK_CFG_W needs to be changed,MBCWordRegionId means which region needs to be changed
bool flash_hiding_test(uint32_t *MBCAddr, uint32_t MBCWordRegionId);

/*******************************************************************************
 * Code
 ******************************************************************************/
int main()
{
    /* Init board hardware. */
    /* attach FRO 12M to FLEXCOMM4 (debug console) */
    CLOCK_SetClkDiv(kCLOCK_DivFlexcom4Clk, 1u);
    CLOCK_AttachClk(BOARD_DEBUG_UART_CLK_ATTACH);

    BOARD_InitPins();
    BOARD_InitBootClocks();
    BOARD_InitDebugConsole();

    TRDC_Type *baseAddr = (TRDC_Type *)TRDC_BASE;

    PRINTF("\r\nFlash Hiding example...\r\n");
    // Hiding for the third 32kb region of the MBC0_DOM_MEM0_BLK_CFG_W0
    // The third 32k region's address range:0x10000 ~ 0x18000
    bool flashHidingTestResult = flash_hiding_test((uint32_t *)(baseAddr->MBC_INDEX->MBC_DOM0_MEM0_BLK_CFG_W), 3);
    if (flashHidingTestResult == true)
    {
        PRINTF("flash hiding test erase and program PASSED\n");
    }
    else
    {
        PRINTF("flash hiding test erase and program FAILED\n");
    }

    PRINTF("\r\nCDPA example...\r\n");
    // CDPA for the fourth 32kb region of the MBC0_DOM_MEM0_BLK_CFG_W0
    // The fourth 32k region's address range:0x18000 ~ 0x20000
    // If the region is set to CDPA, both write and erase operations to this region will result in hardfault
    bool flashCDPATestResult = flash_CDPA_test((uint32_t *)(baseAddr->MBC_INDEX->MBC_DOM0_MEM0_BLK_CFG_W), 4);
    if (flashCDPATestResult)
    {
        PRINTF("CDPA test erase and program PASSED\n");
    }
    else
    {
        PRINTF("CDPA test erase and program FAILED\n");
    }
    while (1)
    {
    }
}

const uint8_t RemapImageData[4096] = {
    0x00, 0xD0, 0x00, 0x20, 0xE1, 0x04, 0x00, 0x00, 0xBF, 0x01, 0x00, 0x00, 0xC3, 0x01, 0x00, 0x00, 0x27, 0x05, 0x00,
    0x00, 0x33, 0x05, 0x00, 0x00, 0xC9, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE0, 0x07, 0x00, 0x00, 0x05, 0x04,
    0x04, 0x00, 0x8D, 0x80, 0xE6, 0xF8, 0xCB, 0x07, 0x00, 0x00, 0xCD, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xCF,
    0x07, 0x00, 0x00, 0xD1, 0x07, 0x00, 0x00, 0xC9, 0x05, 0x00, 0x00, 0xD1, 0x05, 0x00, 0x00, 0xD9, 0x05, 0x00, 0x00,
    0xE1, 0x05, 0x00, 0x00, 0xE9, 0x05, 0x00, 0x00, 0xF1, 0x05, 0x00, 0x00, 0xF9, 0x05, 0x00, 0x00, 0x01, 0x06, 0x00,
    0x00, 0x09, 0x06, 0x00, 0x00, 0x11, 0x06, 0x00, 0x00, 0x19, 0x06, 0x00, 0x00, 0x21, 0x06, 0x00, 0x00, 0x29, 0x06,
    0x00, 0x00, 0x31, 0x06, 0x00, 0x00, 0x39, 0x06, 0x00, 0x00, 0x41, 0x06, 0x00, 0x00, 0x49, 0x06, 0x00, 0x00, 0x51,
    0x06, 0x00, 0x00, 0x59, 0x06, 0x00, 0x00, 0x61, 0x06, 0x00, 0x00, 0x69, 0x06, 0x00, 0x00, 0x71, 0x06, 0x00, 0x00,
    0x79, 0x06, 0x00, 0x00, 0x81, 0x06, 0x00, 0x00, 0x89, 0x06, 0x00, 0x00, 0x91, 0x06, 0x00, 0x00, 0x99, 0x06, 0x00,
    0x00, 0xA1, 0x06, 0x00, 0x00, 0xA9, 0x06, 0x00, 0x00, 0xB1, 0x06, 0x00, 0x00, 0xB9, 0x06, 0x00, 0x00, 0xC1, 0x06,
    0x00, 0x00, 0xC9, 0x06, 0x00, 0x00, 0xD1, 0x06, 0x00, 0x00, 0xD9, 0x06, 0x00, 0x00, 0xE1, 0x06, 0x00, 0x00, 0xE9,
    0x06, 0x00, 0x00, 0xF1, 0x06, 0x00, 0x00, 0xF9, 0x06, 0x00, 0x00, 0x01, 0x07, 0x00, 0x00, 0x09, 0x07, 0x00, 0x00,
    0x11, 0x07, 0x00, 0x00, 0x19, 0x07, 0x00, 0x00, 0x21, 0x07, 0x00, 0x00, 0x29, 0x07, 0x00, 0x00, 0x31, 0x07, 0x00,
    0x00, 0x39, 0x07, 0x00, 0x00, 0x41, 0x07, 0x00, 0x00, 0x49, 0x07, 0x00, 0x00, 0x51, 0x07, 0x00, 0x00, 0x59, 0x07,
    0x00, 0x00, 0x61, 0x07, 0x00, 0x00, 0x69, 0x07, 0x00, 0x00, 0x71, 0x07, 0x00, 0x00, 0x79, 0x07, 0x00, 0x00, 0x81,
    0x07, 0x00, 0x00, 0x89, 0x07, 0x00, 0x00, 0x91, 0x07, 0x00, 0x00, 0x99, 0x07, 0x00, 0x00, 0xA1, 0x07, 0x00, 0x00,
    0xA9, 0x07, 0x00, 0x00, 0xB1, 0x07, 0x00, 0x00, 0xB9, 0x07, 0x00, 0x00, 0xC1, 0x07, 0x00, 0x00, 0xB1, 0x05, 0x00,
    0x00, 0x10, 0xB5, 0x04, 0x46, 0x10, 0x46, 0x0A, 0x46, 0x01, 0x46, 0x20, 0x46, 0x00, 0xF0, 0x02, 0xF8, 0x20, 0x46,
    0x10, 0xBD, 0x62, 0xF3, 0x0F, 0x22, 0x62, 0xF3, 0x1F, 0x42, 0x40, 0x18, 0x10, 0xF0, 0x03, 0x03, 0x08, 0xD0, 0xC9,
    0x1A, 0x1F, 0xD3, 0xDB, 0x07, 0x48, 0xBF, 0x00, 0xF8, 0x01, 0x2D, 0x28, 0xBF, 0x20, 0xF8, 0x02, 0x2D, 0x13, 0x00,
    0x30, 0xB4, 0x14, 0x46, 0x15, 0x46, 0x10, 0x39, 0x28, 0xBF, 0x20, 0xE9, 0x3C, 0x00, 0xFA, 0xD8, 0x49, 0x07, 0x28,
    0xBF, 0x20, 0xE9, 0x0C, 0x00, 0x48, 0xBF, 0x40, 0xF8, 0x04, 0x2D, 0x89, 0x00, 0x28, 0xBF, 0x20, 0xF8, 0x02, 0x2D,
    0x48, 0xBF, 0x00, 0xF8, 0x01, 0x2D, 0x30, 0xBC, 0x70, 0x47, 0xC9, 0x18, 0x18, 0xBF, 0x00, 0xF8, 0x01, 0x2D, 0xCB,
    0x07, 0x28, 0xBF, 0x00, 0xF8, 0x01, 0x2D, 0x70, 0x47, 0xFE, 0xE7, 0x70, 0x47, 0xFE, 0xE7, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08,
    0x20, 0x7B, 0x49, 0x08, 0x60, 0x03, 0x20, 0x7A, 0x49, 0x08, 0x60, 0x70, 0x47, 0x7F, 0xB5, 0x04, 0x00, 0x0D, 0x00,
    0x16, 0x00, 0x0B, 0x22, 0x00, 0x21, 0x68, 0x46, 0xFF, 0xF7, 0x92, 0xFF, 0x00, 0x20, 0x00, 0x2C, 0x10, 0xD1, 0x01,
    0x20, 0x30, 0x21, 0x8D, 0xF8, 0x00, 0x10, 0x0D, 0xE0, 0x0A, 0x21, 0xB4, 0xFB, 0xF1, 0xF2, 0x01, 0xFB, 0x12, 0x41,
    0x30, 0x31, 0x6A, 0x46, 0x11, 0x54, 0x40, 0x1C, 0x0A, 0x21, 0xB4, 0xFB, 0xF1, 0xF4, 0x00, 0x2C, 0xF1, 0xD1, 0x69,
    0x46, 0x01, 0x44, 0x49, 0x1E, 0x00, 0x22, 0x04, 0xE0, 0x0B, 0x78, 0x2B, 0x70, 0x49, 0x1E, 0x6D, 0x1C, 0x52, 0x1C,
    0x82, 0x42, 0xF8, 0xD3, 0x30, 0x60, 0x7F, 0xBD, 0x7C, 0xB5, 0x04, 0x00, 0x0E, 0x00, 0x15, 0x00, 0x08, 0x22, 0x00,
    0x21, 0x68, 0x46, 0xFF, 0xF7, 0x65, 0xFF, 0x00, 0x20, 0x00, 0x2C, 0x10, 0xD1, 0x01, 0x20, 0x30, 0x21, 0x8D, 0xF8,
    0x00, 0x10, 0x30, 0x21, 0x31, 0x70, 0x71, 0x1C, 0x78, 0x22, 0x0A, 0x70, 0x49, 0x1C, 0x6A, 0x46, 0x02, 0x44, 0x52,
    0x1E, 0x00, 0x23, 0x12, 0xE0, 0x57, 0x31, 0x07, 0xE0, 0x00, 0x2C, 0xF0, 0xD0, 0x14, 0xF0, 0x0F, 0x01, 0x24, 0x09,
    0x0A, 0x29, 0xF6, 0xD2, 0x30, 0x31, 0x6A, 0x46, 0x11, 0x54, 0x40, 0x1C, 0xF3, 0xE7, 0x14, 0x78, 0x0C, 0x70, 0x52,
    0x1E, 0x49, 0x1C, 0x5B, 0x1C, 0x83, 0x42, 0xF8, 0xD3, 0x80, 0x1C, 0x28, 0x60, 0x73, 0xBD, 0x0F, 0xB4, 0x78, 0xB5,
    0xC1, 0xB0, 0x46, 0x9C, 0x00, 0x20, 0x00, 0x90, 0x46, 0xAE, 0x36, 0x1D, 0x01, 0xAD, 0x07, 0xE0, 0x20, 0x78, 0x28,
    0x70, 0x6D, 0x1C, 0x02, 0xE0, 0x20, 0x78, 0x28, 0x70, 0x6D, 0x1C, 0x64, 0x1C, 0x20, 0x78, 0x00, 0x28, 0x3A, 0xD0,
    0x20, 0x78, 0x25, 0x28, 0xF1, 0xD1, 0x60, 0x78, 0x64, 0x28, 0x05, 0xD0, 0x60, 0x78, 0x75, 0x28, 0x02, 0xD0, 0x60,
    0x78, 0x6C, 0x28, 0x09, 0xD1, 0x30, 0x68, 0x6A, 0x46, 0x29, 0x00, 0xFF, 0xF7, 0x7A, 0xFF, 0x00, 0x98, 0x05, 0x44,
    0x64, 0x1C, 0x36, 0x1D, 0xE5, 0xE7, 0x60, 0x78, 0x78, 0x28, 0x09, 0xD1, 0x30, 0x68, 0x6A, 0x46, 0x29, 0x00, 0xFF,
    0xF7, 0x9A, 0xFF, 0x00, 0x98, 0x05, 0x44, 0x64, 0x1C, 0x36, 0x1D, 0xD8, 0xE7, 0x60, 0x78, 0x63, 0x28, 0x05, 0xD1,
    0x30, 0x78, 0x28, 0x70, 0x6D, 0x1C, 0x64, 0x1C, 0x36, 0x1D, 0xCF, 0xE7, 0x60, 0x78, 0x73, 0x28, 0xC9, 0xD1, 0x31,
    0x68, 0x03, 0xE0, 0x08, 0x78, 0x28, 0x70, 0x49, 0x1C, 0x6D, 0x1C, 0x08, 0x78, 0x00, 0x28, 0xF8, 0xD1, 0x36, 0x1D,
    0x64, 0x1C, 0xC0, 0xE7, 0x00, 0x20, 0x28, 0x70, 0x01, 0xA9, 0x10, 0x20, 0x1E, 0x4A, 0x10, 0x60, 0x03, 0xE0, 0x08,
    0x78, 0x1D, 0x4A, 0x10, 0x60, 0x49, 0x1C, 0x08, 0x78, 0x00, 0x28, 0xF8, 0xD1, 0x03, 0x20, 0x19, 0x49, 0x08, 0x60,
    0x42, 0xB0, 0x70, 0xBC, 0x5D, 0xF8, 0x14, 0xFB, 0x10, 0xB5, 0x17, 0x48, 0x00, 0x21, 0x00, 0xE0, 0x49, 0x1C, 0x04,
    0x29, 0x0C, 0xD2, 0x0C, 0x22, 0x14, 0x4B, 0x02, 0xFB, 0x01, 0xF4, 0x1C, 0x59, 0xA0, 0x42, 0xF5, 0xD3, 0x02, 0xFB,
    0x01, 0xF2, 0x1A, 0x44, 0x52, 0x68, 0x82, 0x42, 0xEF, 0xD3, 0x05, 0x29, 0x03, 0xD3, 0x0E, 0x48, 0xFF, 0xF7, 0x82,
    0xFF, 0x0B, 0xE0, 0x0B, 0x4A, 0x0C, 0x20, 0x41, 0x43, 0x02, 0xEB, 0x01, 0x00, 0x81, 0x68, 0x0A, 0x48, 0xFF, 0xF7,
    0x78, 0xFF, 0x0A, 0x48, 0xFF, 0xF7, 0x75, 0xFF, 0x09, 0x48, 0xFF, 0xF7, 0x72, 0xFF, 0xFF, 0xF7, 0x09, 0xFF, 0xFE,
    0xE7, 0x00, 0xF2, 0x01, 0x50, 0x04, 0xF2, 0x01, 0x50, 0x99, 0x03, 0x00, 0x00, 0x88, 0x04, 0x00, 0x00, 0x58, 0x04,
    0x00, 0x00, 0xB8, 0x04, 0x00, 0x00, 0x48, 0x05, 0x00, 0x00, 0x94, 0x05, 0x00, 0x00, 0x80, 0xB5, 0x0B, 0x48, 0x01,
    0x68, 0x51, 0xF0, 0x0F, 0x01, 0x01, 0x60, 0x09, 0x49, 0x0A, 0x68, 0x40, 0xF6, 0x03, 0x40, 0x02, 0x43, 0x0A, 0x60,
    0x07, 0x48, 0x08, 0x49, 0x08, 0x60, 0x00, 0x20, 0x07, 0x49, 0x08, 0x60, 0x78, 0x20, 0x07, 0x49, 0x08, 0x60, 0xFF,
    0xF7, 0xC1, 0xFE, 0x01, 0xBD, 0x88, 0xED, 0x00, 0xE0, 0x8C, 0xED, 0x00, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x08, 0xED,
    0x00, 0xE0, 0x08, 0x03, 0x00, 0x50, 0x20, 0x02, 0x00, 0x50, 0x41, 0x70, 0x70, 0x6C, 0x69, 0x63, 0x61, 0x74, 0x69,
    0x6F, 0x6E, 0x20, 0x69, 0x73, 0x20, 0x72, 0x75, 0x6E, 0x6E, 0x69, 0x6E, 0x67, 0x2C, 0x20, 0x75, 0x6E, 0x6B, 0x6E,
    0x6F, 0x77, 0x6E, 0x20, 0x6D, 0x65, 0x6D, 0x6F, 0x72, 0x79, 0x20, 0x72, 0x65, 0x67, 0x69, 0x6F, 0x6E, 0x0A, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xDE, 0x09, 0x00, 0x64, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0xDE,
    0x09, 0x10, 0x74, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0xFF, 0xFF, 0x03, 0x20, 0x84, 0x05, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x30, 0xFF, 0xFF, 0x03, 0x30, 0xA4, 0x05, 0x00, 0x00, 0x41, 0x70, 0x70, 0x6C, 0x69, 0x63, 0x61, 0x74,
    0x69, 0x6F, 0x6E, 0x20, 0x69, 0x73, 0x20, 0x72, 0x75, 0x6E, 0x6E, 0x69, 0x6E, 0x67, 0x20, 0x69, 0x6E, 0x20, 0x25,
    0x73, 0x20, 0x72, 0x65, 0x67, 0x69, 0x6F, 0x6E, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x72, 0xB6, 0x04, 0x48, 0x80, 0xF3,
    0x0A, 0x88, 0x03, 0x48, 0x80, 0x47, 0x62, 0xB6, 0x03, 0x48, 0x00, 0x47, 0x00, 0x00, 0x00, 0xCC, 0x00, 0x20, 0x11,
    0x04, 0x00, 0x00, 0xBD, 0x05, 0x00, 0x00, 0x00, 0xF0, 0x0D, 0xF8, 0x00, 0x28, 0x01, 0xD0, 0xAF, 0xF3, 0x00, 0x80,
    0xAF, 0xF3, 0x00, 0x80, 0x00, 0x20, 0xAF, 0xF3, 0x00, 0x80, 0xFF, 0xF7, 0x3F, 0xFF, 0x00, 0xF0, 0x02, 0xF8, 0x01,
    0x20, 0x70, 0x47, 0x00, 0xF0, 0x01, 0xB8, 0xFE, 0xE7, 0x07, 0x46, 0x38, 0x46, 0x00, 0xF0, 0x02, 0xF8, 0xFB, 0xE7,
    0xFE, 0xE7, 0x80, 0xB5, 0xAF, 0xF3, 0x00, 0x80, 0x02, 0x4A, 0x11, 0x00, 0x18, 0x20, 0xAB, 0xBE, 0xFB, 0xE7, 0x26,
    0x00, 0x02, 0x00, 0x66, 0x6C, 0x65, 0x78, 0x73, 0x70, 0x69, 0x20, 0x75, 0x70, 0x64, 0x61, 0x74, 0x65, 0x20, 0x69,
    0x6D, 0x61, 0x67, 0x65, 0x20, 0x50, 0x41, 0x53, 0x53, 0x21, 0x0A, 0x00, 0x66, 0x6C, 0x61, 0x73, 0x68, 0x5F, 0x6E,
    0x6F, 0x6E, 0x73, 0x65, 0x63, 0x75, 0x72, 0x65, 0x00, 0x66, 0x6C, 0x61, 0x73, 0x68, 0x5F, 0x73, 0x65, 0x63, 0x75,
    0x72, 0x65, 0x00, 0x00, 0x00, 0x00, 0x72, 0x61, 0x6D, 0x5F, 0x6E, 0x6F, 0x6E, 0x73, 0x65, 0x63, 0x75, 0x72, 0x65,
    0x00, 0x00, 0x00, 0x69, 0x6D, 0x61, 0x67, 0x65, 0x31, 0x20, 0x50, 0x41, 0x53, 0x53, 0x21, 0x0A, 0x00, 0x00, 0x00,
    0x72, 0x61, 0x6D, 0x5F, 0x73, 0x65, 0x63, 0x75, 0x72, 0x65, 0x00, 0x00, 0x01, 0x48, 0x00, 0x47, 0xFE, 0xE7, 0x00,
    0x00, 0xB5, 0x05, 0x00, 0x00, 0xAF, 0xF3, 0x00, 0x80, 0xAF, 0xF3, 0x00, 0x80, 0xFF, 0xF7, 0x9C, 0xFF, 0x00, 0x48,
    0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5,
    0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00,
    0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00,
    0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05,
    0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00,
    0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47,
    0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00,
    0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48,
    0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5,
    0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00,
    0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00,
    0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05,
    0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00,
    0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47,
    0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00,
    0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48,
    0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5,
    0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00,
    0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00,
    0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05,
    0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00,
    0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47,
    0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00,
    0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48,
    0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5,
    0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00,
    0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0x00, 0x48, 0x00, 0x47, 0xB5, 0x05, 0x00, 0x00, 0xFE, 0xE7, 0xFE,
    0xE7, 0xFE, 0xE7, 0xFE, 0xE7, 0xFE, 0xE7, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00};

bool flash_CDPA_test(uint32_t *CDPABlock, uint32_t CDPAPageId)
{
    *CDPABlock        = 0x0;
    bool testResult   = false;
    status_t status   = kStatus_Success;
    uint32_t CDPAaddr = 0x18000;
    uint32_t readBuffer[0x200];

    do
    {
        FlashConfig2.modeConfig.sysFreqInMHz = 96;

        status = FLASH_Init(&FlashConfig2);
        if (status != kStatus_FLASH_Success)
        {
            PRINTF("FLASH_Init FAILED\n");
            break;
        }
        else
        {
            PRINTF("FLASH_Init PASSED\n");
        }

        status = FFR_Init(&FlashConfig2);
        if (status != kStatus_FLASH_Success)
        {
            PRINTF("FFR_Init FAILED\n");
            break;
        }
        else
        {
            PRINTF("FFR_Init PASSED\n");
        }

        PRINTF("Before enabling the CDPA, call flash_erase API and flash_program API ...\n");
        PRINTF("Erasing FLASH sectors, address: %x, size:%x\n", CDPAaddr, REMAP_IMAGE_SIZE);
        status = FLASH_Erase(&FlashConfig2, CDPAaddr, REMAP_IMAGE_SIZE, kFLASH_ApiEraseKey);
        if (status != kStatus_Success)
        {
            PRINTF("flash_page_erase returned error:%d\n", status);
            return testResult;
        }
        else
        {
            PRINTF("flash_page_erase PASSED\n");
        }

        PRINTF("Programming FLASH, address: %x, size: %x\n", CDPAaddr, sizeof(RemapImageData));
        status = FLASH_Program(&FlashConfig2, CDPAaddr, (uint8_t *)&RemapImageData, sizeof(RemapImageData));
        if (status != kStatus_Success)
        {
            PRINTF("flash_program returned error:%d\n", status);
            return testResult;
        }
        else
        {
            PRINTF("CDPA no enabled and flash_program PASSED\n");
        }

        PRINTF("CDPABlock = %x\n", *CDPABlock);
        *CDPABlock = 0x4 << ((CDPAPageId - 1) * 4);
        PRINTF("CDPABlock = %x\n", *CDPABlock);

        status = FLASH_Read(&FlashConfig1, CDPAaddr, (uint8_t *)readBuffer, sizeof(readBuffer));
        if (status != kStatus_Success)
        {
            PRINTF("read FAILED\n");
            break;
        }
        else
        {
            PRINTF("CDPA enabled and read test PASSED\n");
        }
        testResult = true;

    } while (0);

    return testResult;
}

bool flash_hiding_test(uint32_t *MBCAddr, uint32_t MBCWordRegionId)
{
    bool testResult          = false;
    status_t status          = kStatus_Success;
    uint32_t flashHidingaddr = 0x10000;

    uint32_t readBuffer[0x200];

    do
    {
        FlashConfig1.modeConfig.sysFreqInMHz = 96;

        status = FLASH_Init(&FlashConfig1);
        if (status != kStatus_FLASH_Success)
        {
            PRINTF("FLASH_Init FAILED\n");
            break;
        }
        else
        {
            PRINTF("FLASH_Init PASSED\n");
        }
        status = FFR_Init(&FlashConfig1);
        if (status != kStatus_FLASH_Success)
        {
            PRINTF("FFR_Init FAILED\n");
            break;
        }
        else
        {
            PRINTF("FFR_Init PASSED\n");
        }

        PRINTF("Erasing FLASH sectors, address: %x, size:%x\n", flashHidingaddr, REMAP_IMAGE_SIZE);
        status = FLASH_Erase(&FlashConfig1, flashHidingaddr, REMAP_IMAGE_SIZE, kFLASH_ApiEraseKey);
        if (status != kStatus_Success)
        {
            PRINTF("flash_page_erase returned error:%d\n", status);
            return testResult;
        }
        else
        {
            PRINTF("flash_page_erase PASSED\n");
        }

        PRINTF("Programming FLASH, address: %x, size: %x\n", flashHidingaddr, sizeof(RemapImageData));
        status = FLASH_Program(&FlashConfig1, flashHidingaddr, (uint8_t *)&RemapImageData, sizeof(RemapImageData));
        if (status != kStatus_Success)
        {
            PRINTF("flash_page_program returned error:%d\n", status);
            return testResult;
        }
        else
        {
            PRINTF("flash_page_program PASSED\n");
        }

        status = FLASH_Read(&FlashConfig1, flashHidingaddr, (uint8_t *)readBuffer, sizeof(readBuffer));
        if (status == kStatus_FLASH_ReadHidingAreaDisallowed)
        {
            PRINTF("FLASH hiding enabled and read linmit!\n");
            break;
        }
        else
        {
            PRINTF("FLASH hiding no enabled and read PASSED!\n");
        }
        PRINTF("MBCAddr = %x\n", *MBCAddr);
        *MBCAddr = 0x5 << ((MBCWordRegionId - 1) * 4);
        PRINTF("MBCAddr = %x\n", *MBCAddr);
        status = FLASH_Read(&FlashConfig1, flashHidingaddr, (uint8_t *)readBuffer, sizeof(readBuffer));
        if (status != kStatus_FLASH_ReadHidingAreaDisallowed)
        {
            PRINTF("FLASH hiding disabled and read test FAILED\n");
            break;
        }
        else
        {
            PRINTF("FLASH hiding enabled and read limit PASSED\n");
        }

        testResult = true;

    } while (0);

    return testResult;
}
